/*
 *	wiic
 *
 *	This file is part of WiiC, written by:
 *		Gabriele Randelli
 *		Email: randelli@dis.uniroma1.it
 *
 *	Copyright 2010
 *
 *	This file is based on Wiiuse, written By:
 *		Michael Laforest	< para >
 *		Email: < thepara (--AT--) g m a i l [--DOT--] com >
 *
 *	Copyright 2006-2007
 *
 *	This file is part of wiic.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *	$Header$
 *
 */

/**
 *	@file
 *
 *	@brief Example using the wiic API.
 *
 *	This file is an example of how to use the wiic library.
 */
#include "bcm2835.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <wiic.h>

#define MAX_WIIMOTES	4


#define A0 RPI_BPLUS_GPIO_J8_03
#define A1 RPI_BPLUS_GPIO_J8_05
#define A2 RPI_BPLUS_GPIO_J8_07
#define A3 RPI_BPLUS_GPIO_J8_29

int exiting = 0;

/**
 *	@brief Callback that handles an event.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *
 *	This function is called automatically by the wiic library when an
 *	event occurs on the specified wiimote.
 */
void handle_event(struct wiimote_t* wm) {
	printf("\n\n--- EVENT [id %i] ---\n", wm->unid);

	/* if a button is pressed, report it */
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_A))		printf("A pressed\n");
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_B))		printf("B pressed\n");
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_UP))		printf("UP pressed\n");
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_DOWN))	printf("DOWN pressed\n");
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_LEFT))	printf("LEFT pressed\n");
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_RIGHT))	printf("RIGHT pressed\n");
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_MINUS))	printf("MINUS pressed\n");
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_PLUS))	printf("PLUS pressed\n");
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_ONE))		printf("ONE pressed\n");
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_TWO))		printf("TWO pressed\n");
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_HOME))	printf("HOME pressed\n");



	if(IS_PRESSED(wm, WIIMOTE_BUTTON_LEFT))
	    bcm2835_gpio_write_mask(0b010000, (1 << A0) | (1 << A1)| (1 << A2)| (1 << A3));
		//wiic_set_motion_plus(wm,0);
	if(IS_PRESSED(wm, WIIMOTE_BUTTON_RIGHT))
	    bcm2835_gpio_write_mask(0b000100, (1 << A0) | (1 << A1)| (1 << A2)| (1 << A3));
		//wiic_set_motion_plus(wm,1);
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_UP))
	    bcm2835_gpio_write_mask(0b010100, (1 << A0) | (1 << A1)| (1 << A2)| (1 << A3));
		//wiic_set_ir(wm, 1);
	if (IS_PRESSED(wm, WIIMOTE_BUTTON_DOWN))
	    bcm2835_gpio_write_mask(0b101000, (1 << A0) | (1 << A1)| (1 << A2)| (1 << A3));
		//wiic_set_ir(wm, 0);

	if(IS_RELEASED(wm, WIIMOTE_BUTTON_LEFT))
	    bcm2835_gpio_write_mask(0b000000, (1 << A0) | (1 << A1)| (1 << A2)| (1 << A3));
		//wiic_set_motion_plus(wm,0);
	if(IS_RELEASED(wm, WIIMOTE_BUTTON_RIGHT))
	    bcm2835_gpio_write_mask(0b000000, (1 << A0) | (1 << A1)| (1 << A2)| (1 << A3));
		//wiic_set_motion_plus(wm,1);
	if (IS_RELEASED(wm, WIIMOTE_BUTTON_UP))
	    bcm2835_gpio_write_mask(0b000000, (1 << A0) | (1 << A1)| (1 << A2)| (1 << A3));
		//wiic_set_ir(wm, 1);
	if (IS_RELEASED(wm, WIIMOTE_BUTTON_DOWN))
	    bcm2835_gpio_write_mask(0b000000, (1 << A0) | (1 << A1)| (1 << A2)| (1 << A3));
		//wiic_set_ir(wm, 0);
}


/**
 *	@brief Callback that handles a read event.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param data		Pointer to the filled data block.
 *	@param len		Length in bytes of the data block.
 *
 *	This function is called automatically by the wiic library when
 *	the wiimote has returned the full data requested by a previous
 *	call to wiic_read_data().
 *
 *	You can read data on the wiimote, such as Mii data, if
 *	you know the offset address and the length.
 *
 *	The \a data pointer was specified on the call to wiic_read_data().
 *	At the time of this function being called, it is not safe to deallocate
 *	this buffer.
 */
void handle_read(struct wiimote_t* wm, byte* data, unsigned short len) {
	int i = 0;

	printf("\n\n--- DATA READ [wiimote id %i] ---\n", wm->unid);
	printf("finished read of size %i\n", len);
	for (; i < len; ++i) {
		if (!(i%16))
			printf("\n");
		printf("%x ", data[i]);
	}
	printf("\n\n");
}


/**
 *	@brief Callback that handles a controller status event.
 *
 *	@param wm				Pointer to a wiimote_t structure.
 *	@param attachment		Is there an attachment? (1 for yes, 0 for no)
 *	@param speaker			Is the speaker enabled? (1 for yes, 0 for no)
 *	@param ir				Is the IR support enabled? (1 for yes, 0 for no)
 *	@param led				What LEDs are lit.
 *	@param battery_level	Battery level, between 0.0 (0%) and 1.0 (100%).
 *
 *	This occurs when either the controller status changed
 *	or the controller status was requested explicitly by
 *	wiic_status().
 *
 *	One reason the status can change is if the nunchuk was
 *	inserted or removed from the expansion port.
 */
void handle_ctrl_status(struct wiimote_t* wm) {
	printf("\n\n--- CONTROLLER STATUS [wiimote id %i] ---\n", wm->unid);

	printf("attachment:      %i\n", wm->exp.type);
	printf("speaker:         %i\n", WIIC_USING_SPEAKER(wm));
	printf("ir:              %i\n", WIIC_USING_IR(wm));
	printf("leds:            %i %i %i %i\n", WIIC_IS_LED_SET(wm, 1), WIIC_IS_LED_SET(wm, 2), WIIC_IS_LED_SET(wm, 3), WIIC_IS_LED_SET(wm, 4));
	printf("battery:         %f %%\n", wm->battery_level);
}


/**
 *	@brief Callback that handles a disconnection event.
 *
 *	@param wm				Pointer to a wiimote_t structure.
 *
 *	This can happen if the POWER button is pressed, or
 *	if the connection is interrupted.
 */
void handle_disconnect(wiimote* wm) {
	printf("\n\n--- DISCONNECTED [wiimote id %i] ---\n", wm->unid);
	exiting = 1;
}


void test(struct wiimote_t* wm, byte* data, unsigned short len) {
	printf("test: %i [%x %x %x %x]\n", len, data[0], data[1], data[2], data[3]);
}



/**
 *	@brief main()
 *
 *	Connect to up to two wiimotes and print any events
 *	that occur on either device.
 */
int main(int argc, char** argv) {
	wiimote** wiimotes;
	int found, connected;

	if (!bcm2835_init()) return 1;

	// Set RPI pin  to be an ouput
    bcm2835_gpio_fsel(A0,     BCM2835_GPIO_FSEL_OUTP  );
    bcm2835_gpio_fsel(A1,     BCM2835_GPIO_FSEL_OUTP  );
    bcm2835_gpio_fsel(A2,     BCM2835_GPIO_FSEL_OUTP  );
    bcm2835_gpio_fsel(A3,     BCM2835_GPIO_FSEL_OUTP  );
//set rpi pin to low
    bcm2835_gpio_clr(A0);
    bcm2835_gpio_clr(A1);
    bcm2835_gpio_clr(A2);
    bcm2835_gpio_clr(A3);


	/*
	 *	Initialize an array of wiimote objects.
	 *
	 *	The parameter is the number of wiimotes I want to create.
	 */

	wiimotes =  wiic_init(MAX_WIIMOTES);

	/*
	 *	Find wiimote devices
	 *
	 *	Now we need to find some wiimotes.
	 *	Give the function the wiimote array we created, and tell it there
	 *	are MAX_WIIMOTES wiimotes we are interested in.
	 *
	 *	Set the timeout to be 5 seconds.
	 *
	 *	This will return the number of actual wiimotes that are in discovery mode.
	 */


	found = wiic_find(wiimotes, MAX_WIIMOTES, 5);
	if (!found) {
		printf ("No wiimotes found.");
		return 0;
	}

	/*
	 *	Connect to the wiimotes
	 *
	 *	Now that we found some wiimotes, connect to them.
	 *	Give the function the wiimote array and the number
	 *	of wiimote devices we found.
	 *
	 *	This will return the number of established connections to the found wiimotes.
	 */
	connected = wiic_connect(wiimotes, found);
	if (connected)
		printf("Connected to %i wiimotes (of %i found).\n", connected, found);
	else {
		printf("Failed to connect to any wiimote.\n");
		return 0;
	}

	/*
	 *	Now set the LEDs and rumble for a second so it's easy
	 *	to tell which wiimotes are connected (just like the wii does).
	 */
	wiic_set_leds(wiimotes[0], WIIMOTE_LED_1);
	wiic_rumble(wiimotes[0], 1);
	usleep(200000);
	wiic_rumble(wiimotes[0], 0);

	/*
	 *	Maybe I'm interested in the battery power of the 0th
	 *	wiimote.  This should be WIIMOTE_ID_1 but to be sure
	 *	you can get the wiimote assoicated with WIIMOTE_ID_1
	 *	using the wiic_get_by_id() function.
	 *
	 *	A status request will return other things too, like
	 *	if any expansions are plugged into the wiimote or
	 *	what LEDs are lit.
	 */
	wiic_status(wiimotes[0]);
/*
	printf("\nPress PLUS (MINUS) to enable (disable) Motion Sensing Report (only accelerometers)\n");
	printf("Press RIGHT (LEFT) to enable (disable) Motion Plus (requires Motion Sensing enabled)\n");
	printf("Press UP (DOWN) to enable (disable) IR camera (requires some IR led)\n");
*/
	/*
	 *	This is the main loop
	 *
	 *	wiic_poll() needs to be called with the wiimote array
	 *	and the number of wiimote structures in that array
	 *	(it doesn't matter if some of those wiimotes are not used
	 *	or are not connected).
	 *
	 *	This function will set the event flag for each wiimote
	 *	when the wiimote has things to report.
	 */
	while (!exiting) {
		if (wiic_poll(wiimotes, connected)) {
			/*
			 *	This happens if something happened on any wiimote.
			 *	So go through each one and check if anything happened.
			 */
			int i = 0;
			for (; i < MAX_WIIMOTES; ++i) {
				switch (wiimotes[i]->event) {
					case WIIC_EVENT:
						// a generic event occured
						handle_event(wiimotes[i]);
						break;

					case WIIC_STATUS:
						// a status event occured
						handle_ctrl_status(wiimotes[i]);
						break;

					case WIIC_DISCONNECT:
					case WIIC_UNEXPECTED_DISCONNECT:
						// the wiimote disconnected
						handle_disconnect(wiimotes[i]);
						break;

					case WIIC_READ_DATA:
						/*
						 *	Data we requested to read was returned.
						 *	Take a look at wiimotes[i]->read_req
						 *	for the data.
						 */
						break;

					case WIIC_NUNCHUK_INSERTED:
						/*
						 *	a nunchuk was inserted
						 *	This is a good place to set any nunchuk specific
						 *	threshold values.  By default they are the same
						 *	as the wiimote.
						 */
						printf("Nunchuk inserted.\n");
						break;

					case WIIC_CLASSIC_CTRL_INSERTED:
						printf("Classic controller inserted.\n");
						break;

					case WIIC_GUITAR_HERO_3_CTRL_INSERTED:
						// some expansion was inserted
						handle_ctrl_status(wiimotes[i]);
						break;

					case WIIC_MOTION_PLUS_INSERTED:
						printf("Motion Plus inserted.\n");
						break;

					case WIIC_BALANCE_BOARD_INSERTED:
						printf("Balance Board connected.\n");
						break;

					case WIIC_BALANCE_BOARD_REMOVED:
						printf("Balance Board disconnected.\n");
						break;

					case WIIC_NUNCHUK_REMOVED:
					case WIIC_CLASSIC_CTRL_REMOVED:
					case WIIC_GUITAR_HERO_3_CTRL_REMOVED:
					case WIIC_MOTION_PLUS_REMOVED:
						// some expansion was removed
						handle_ctrl_status(wiimotes[i]);
						printf("An expansion was removed.\n");
						break;

					default:
						break;
				}
			}
		}
	}

	/*
	 *	Disconnect the wiimotes
	 */
	wiic_cleanup(wiimotes, MAX_WIIMOTES);

	return 0;
}
